%%%%% solving the optimal growth model with collocation methods

clearvars;
close all;
clc;

%%%%% economic parameters

alpha = 1/3;        %% capital's share in production function
beta  = 0.95;       %% time discount factor   
delta = 0.05;       %% depreciation rate
sigma = 1;          %% CRRA (=1/IES)
rho   = (1/beta)-1; %% implied rate of time preference

kstar = (alpha/(rho+delta))^(1/(1-alpha)); %% steady state
kbar  = (1/delta)^(1/(1-alpha));

%%%%% put in a structure to pass to other functions

parameters.alpha = alpha;
parameters.beta  = beta;
parameters.delta = delta;
parameters.sigma = sigma;

%%%%% numerical parameters

max_iter = 500;      %% maximum number of iterations
tol      = 1e-7;     %% treat numbers smaller than this as zero

do_newton = 1;       %% otherwise does function iteration

%%%%% set up grid of capital stock

n        = 99;       %% number of breakpoints for k grid
kmin     = tol;      %% effectively zero 
kmax     = kbar;     %% effective upper bound

curv     = 0.5;      %% (curv = 0 log-spaced, curv = 1 linear)
breaks   = nodeunif(n, kmin.^curv, kmax.^curv).^(1/curv);

N        = 1001;     %% number of nodes for fine grid (see below)

%%%%% bounds on consumption

cmin     = tol;
cmax     = (kmax.^alpha) + (1-delta)*kmax - eps;

%%%%% setup state space using CompEcon tools

fspace  = fundef({'spli', breaks}); %% function space structure


grid    = funnode(fspace); % nodes where we solve the problem
Phi     = funbas(fspace);  % matrix of collocation basis vectors 
                           % Phi_{ij} = phi_j(k_i)

k       = grid;            % grid has n+2 elements

kmin    = k(1);
kmax    = k(end);
                           

%%%%% initial guess at collocation coefficients "a"

c     = alpha*beta*k.^alpha; % guess for consumption policy
v     = log(c)/(1-beta);     % guess for value function

a     = Phi\v;               % implied collocation coefficients

tic
%%%%% solve Bellman equation

for i=1:max_iter;
 
%%%%% optimal consumption given coefficients "a"

c = solve_brent('rhs_bellman',k,parameters,a,fspace,cmin,cmax,tol);

%%%%% maximized rhs of Bellman equation

v = rhs_bellman(c,k,parameters,a,fspace); %% v(a)

%%%%% updated collocation coefficients

if do_newton == 1,

%%%%% implied by optimal consumption    
kprime   = (k.^alpha) + (1-delta)*k-c; 

%%%%% Jacobian matrix of v(a)
Jacobian = beta*funbas(fspace,kprime);   

%%%%% Newton's method
anew     = a - (Phi-Jacobian)\(Phi*a-v);    
    
else    
    
anew     = Phi\v;

end

%%%%% check if converged

error = norm(anew-a,inf);

fprintf('%4i %6.2e \n',[i, error]);

if error<tol, break, end;

%%%%% if not converged, update and try again

a = anew;

end

time      = toc;
last_iter = i;

if error<tol && last_iter<=max_iter,
   
fprintf('\n');    
fprintf('time taken (seconds) = %3.3f \n',time);
fprintf('number of iterations = %3.0f \n',last_iter);
fprintf('error at last step   = %3.10f \n',error);
fprintf('\n'); 

end


%%%%% interpolate on finer grid

%%%%% finer grid
kfine = nodeunif(N, kmin.^curv, kmax.^curv).^(1/curv);    

%%%%% interpolated value function
vfine = funeval(a,fspace,kfine); 

%%%%% interpolation coefficients for c(k) 
ac    = funfitxy(fspace,k,c);     

%%%%% interpolated policy functions
cfine = funeval(ac,fspace,kfine);
gfine = kfine.^alpha+(1-delta)*kfine - cfine;

figure(1)
plot(kfine,vfine,'b-')
xlabel('capital stock, k')
ylabel('value function, v(k)')

figure(2)
plot(kfine,gfine,'r-',k,k,'k--')
xlabel('capital stock, k')
ylabel('policy function, g(k)')



